#!/bin/sh

ACTION=${1}
shift 1

get_image() {
  IMAGE_NAME=$(uci get nextcloud.@nextcloud[0].image_name 2>/dev/null)
}

do_install_detail() {
  local config=$(uci get nextcloud.@nextcloud[0].config_path 2>/dev/null)
  local port=$(uci get nextcloud.@nextcloud[0].port 2>/dev/null)
  local IMAGE_NAME=$(uci get nextcloud.@nextcloud[0].image_name 2>/dev/null)

  GEN_PASS=$(< /dev/urandom tr -dc A-Za-z0-9 2>/dev/null | head -c14; echo)
  GEN_PASS2=$(< /dev/urandom tr -dc A-Za-z0-9 2>/dev/null | head -c14; echo)

  LAN_IP=$(uci get network.lan.ipaddr)
  LAN_IP="${LAN_IP%/*}"

  if [ -z "$config" ]; then
    echo "config path is empty!"
    exit 1
  fi

  [ -z "$port" ] && port=8531
  [ -z "$IMAGE_NAME" ] && IMAGE_NAME=nextcloud:latest

  rm -R /opt/docker2/compose/nextcloud-app
  mkdir /opt/docker2/compose/nextcloud-app

  touch /opt/docker2/compose/nextcloud-app/docker-compose.yml
  cat > /opt/docker2/compose/nextcloud-app/docker-compose.yml <<EOF
version: '3.7'
services:
  app:
    image: $IMAGE_NAME
    restart: always
    ports:
      - ${LAN_IP}:${port}:80
    volumes:
      - /opt/docker2/compose/nextcloud-app/data:/var/www/html
    environment:
      - MYSQL_PASSWORD=${GEN_PASS}
      - MYSQL_DATABASE=nextcloud
      - MYSQL_USER=nextcloud
      - MYSQL_HOST=db
    labels:
      plugsy.name: "Nextcloud"
      plugsy.category: "Home"
      plugsy.link: "http://${LAN_IP}:${port}"

  db:
    image: mariadb:10.5
    restart: always
    volumes:
      - db:/var/lib/mysql
    environment:
      - MYSQL_ROOT_PASSWORD=${GEN_PASS}
      - MYSQL_PASSWORD=${GEN_PASS}
      - MYSQL_DATABASE=nextcloud
      - MYSQL_USER=nextcloud
EOF

  docker-compose -f /opt/docker2/compose/nextcloud-app/docker-compose.yml up -d

  uci add shortcutmenu lists
  uci set shortcutmenu.@lists[-1].webname="$IMAGE_NAME"
  uci set shortcutmenu.@lists[-1].weburl="$LAN_IP:$port"
  uci set shortcutmenu.@lists[-1].webpath="/"
  uci commit shortcutmenu
}

usage() {
  echo "usage: $0 sub-command"
  echo "where sub-command is one of:"
  echo "      install                Install Nextcloud"
  echo "      upgrade                Upgrade Nextcloud"
  echo "      rm/start/stop/restart  Remove/Start/Stop/Restart Nextcloud"
  echo "      status                 Nextcloud status"
  echo "      port                   Nextcloud port"
}

case ${ACTION} in
  "install"|"upgrade")
    get_image
    do_install_detail
  ;;
  "rm")
    get_image
    CONTAINER_IDS=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.ID}}')
    echo "Stopping and removing containers..."
    for ID in $CONTAINER_IDS; do
      docker stop $ID
      docker rm $ID
    done
    docker rmi -f $IMAGE_NAME
  ;;
  "start"|"stop"|"restart")
    get_image
    CONTAINER_IDS=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.ID}}')
    for ID in $CONTAINER_IDS; do
      docker ${ACTION} ${ID}
    done
  ;;
  "status")
    get_image
    CONTAINER_NAMES=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.Names}}')
    docker ps --all -f "name=${CONTAINER_NAMES}" --format '{{.Status}}'
  ;;
  "port")
    get_image
    CONTAINER_NAMES=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.Names}}')
    docker ps --all -f "name=${CONTAINER_NAMES}" --format '{{.Ports}}' | grep -om1 '0.0.0.0:[0-9]*' | sed 's/0.0.0.0://'
  ;;
  *)
    usage
    exit 1
  ;;
esac
